/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.field;

import java.awt.Container;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import javax.swing.JComponent;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import nl.cloudfarming.client.util.actions.OpenCloseHandler;
import org.netbeans.api.actions.Openable;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.Utilities;
import org.openide.windows.TopComponent;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public abstract class BaseFieldTaskElement extends TopComponent implements MultiViewElement {

    protected final FieldTask fieldTask;
    private final Lookup.Result<FieldTask> fieldTaskResult = Utilities.actionsGlobalContext().lookupResult(FieldTask.class);
    private static final Logger LOGGER = Logger.getLogger(BaseFieldTaskElement.class.getName());
    //
    // MultiViewElement
    //
    private JToolBar toolbar = new JToolBar();
    private transient MultiViewElementCallback callback;
    private final LookupListener fieldTaskListener = new LookupListener() {
        @Override
        public void resultChanged(LookupEvent ev) {
            Collection<FieldTask> tasks = (Collection<FieldTask>) ((Lookup.Result<FieldTask>) ev.getSource()).allInstances();
            for (FieldTask task : tasks) {
                if (task.equals(fieldTask)) {
                    SwingUtilities.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            Openable openable = getLookup().lookup(Openable.class);
                            if (openable != null) {
                                openable.open();
                            }
                        }
                    });

                }
            }
        }
    };

    //
    // state manegement
    //
    private enum State {

        OPEN, CLOSED
    };
    private static final Map<Container, State> states = Collections.synchronizedMap(new HashMap<Container, State>());

    public BaseFieldTaskElement(Lookup lookup) {
        super(lookup);
        this.fieldTask = lookup.lookup(FieldTask.class);;
    }

    @Override
    public void addNotify() {
        super.addNotify();
        synchronized (states) {
            if (!states.containsKey(getParent())
                    || State.CLOSED.equals(states.get(getParent()))) {
                fieldTaskResult.addLookupListener(fieldTaskListener);
                OpenCloseHandler.Event.triggerOpen(getLookup().lookup(Node.class));
                states.put(getParent(), State.OPEN);
            }
        }
    }

    @Override
    public void removeNotify() {
        super.removeNotify();
        synchronized (states) {
            if (State.OPEN.equals(states.get(getParent()))) {
                fieldTaskResult.removeLookupListener(fieldTaskListener);
                OpenCloseHandler.Event.triggerClose(getLookup().lookup(Node.class));
                states.put(getParent(), State.CLOSED);
            }
        }
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentClosed() {
        removeNotify();
    }

    @Override
    public void componentOpened() {
        addNotify();
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    public void updateName() {
        if (callback != null) {
            String name = fieldTask.getName();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
            tc.setName(getName());
            tc.setToolTipText(getToolTipText());
        }
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }
}
