/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.field;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import java.awt.Image;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import nl.cloudfarming.client.field.model.ActivityField;
import nl.cloudfarming.client.geoviewer.Geographical;
import nl.cloudfarming.client.model.ItemIdType;
import nl.cloudfarming.client.task.ExportableTask;
import nl.cloudfarming.client.util.UOM;
import org.geotools.coverage.grid.GridCoverage2D;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 * @author Maciek Dulkiewicz
 */
public class FieldTask extends ExportableTask {

    public static final double MIN_WORK_WIDTH = 5;
    private String unit;
    public static final String MIME_TYPE = "application/x-agrosense-field-task";
    public static final String EXT = "ftk";
    public static final String PROP_TREATMENTZONE_VALUE = "treatmentZoneValue";
    public static final String PROP_UOM = "uom";

    private double workWidth = MIN_WORK_WIDTH;
    private UOM uom;

    public FieldTask(ActivityField activityField) {
        super(activityField.getURI(), new Class[]{ActivityField.class}, ItemIdType.FTK);
        ActivityField copy = new ActivityField(activityField);
        getContext().addItem(copy);
        setName(activityField.getName());
        recalculateAverageTznValue();
        ic.add(new FieldTaskGeometrical(activityField));
    }

    public String getUnit() {
        return unit;
    }

    public void setUnit(String unit) {
        this.unit = unit;
    }

    @Override
    public double getWorkWidth() {
        return workWidth;
    }

    public void setWorkWidth(double workWidth) {
        double oldWorkWidth = this.workWidth;
        this.workWidth = workWidth;
        setRegenerateGrid(true);
        getPropertyChangeSupport().firePropertyChange(ExportableTask.PROP_WORK_WIDTH, oldWorkWidth, this.workWidth);
    }

    public void fireExternalPropertyChange(String eventName, Object oldValue, Object newValue) {
        getPropertyChangeSupport().firePropertyChange(eventName, oldValue, newValue);
    }

    public UOM getUom() {
        return uom;
    }

    public void setUom(UOM uom) {
        UOM old = this.uom;
        this.uom = uom;
        getPropertyChangeSupport().firePropertyChange(PROP_UOM, old, uom);
    }
    
    protected class FieldTaskGeometrical implements Geographical<GridCoverage2D>, Serializable {

        private ActivityField activityField;

        public FieldTaskGeometrical(ActivityField activityField) {
            this.activityField = activityField;
        }

        @Override
        public Point getCentroid() {
            return activityField.getCentroid();
        }

        @Override
        public Geometry getBoundingBox() {
            return activityField.getBoundingBox();
        }

        @Override
        public GridCoverage2D getRenderObject(Envelope envelope) {
            return FieldTask.this.getExportPreviewGrid();
        }

        @Override
        public String getTooltipText() {
            return activityField.getTooltipText();
        }

        @Override
        public Image getIcon() {
            return activityField.getIcon();
        }

        @Override
        public String getIconLabel() {
            return activityField.getIconLabel();
        }

        @Override
        public void addPropertyChangeListener(PropertyChangeListener listener) {
            activityField.addPropertyChangeListener(listener);
        }

        @Override
        public void removePropertyChangeListener(PropertyChangeListener listener) {
            activityField.removePropertyChangeListener(listener);
        }
    }
}
