/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.field;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.FocusEvent;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;
import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.DefaultCaret;
import javax.swing.text.JTextComponent;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.field.model.TreatmentZone;
import nl.cloudfarming.client.task.ExportableTask;
import nl.cloudfarming.client.task.Task;
import nl.cloudfarming.client.util.swing.BodyPanel;
import nl.cloudfarming.client.util.swing.FormRow;
import nl.cloudfarming.client.util.swing.UnifiedDocumentListener;
import nl.cloudfarming.client.util.swing.beanbinding.DoubleConverter;
import nl.cloudfarming.client.util.swing.beanbinding.DoubleValidator;
import nl.cloudfarming.client.util.swing.beanbinding.NonEmptyStringValidator;
import nl.cloudfarming.client.util.swing.beanbinding.PositiveDoubleValidator;
import org.jdesktop.beansbinding.AutoBinding;
import org.jdesktop.beansbinding.BeanProperty;
import org.jdesktop.beansbinding.Binding;
import org.jdesktop.beansbinding.BindingGroup;
import org.jdesktop.beansbinding.Bindings;
import org.jdesktop.beansbinding.ELProperty;
import org.jdesktop.beansbinding.Validator;
import org.jdesktop.swingbinding.JTableBinding;
import org.jdesktop.swingbinding.SwingBindings;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.Lookups;
import org.openide.windows.TopComponent;

/**
 * Data panel for tasks Displays a name field and a table with unique values for
 * TreatmentZones
 *
 * Not currently used in AgroSense but in isomatch
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@TopComponent.Description(preferredID = "com.kverneland.isomatch.workflow.windows.FieldTaskDataElement",
iconBase = "com/kverneland/isomatch/workflow/to_do_list_checked_1.png",
persistenceType = TopComponent.PERSISTENCE_NEVER)
@TopComponent.Registration(mode = "properties", openAtStartup = false)
@MultiViewElement.Registration(displayName = "#FieldTaskDataElement_display_name",
iconBase = "nl/cloudfarming/client/task/field/to_do_list_checked_1.png",
mimeType = FieldTask.MIME_TYPE,
preferredID = "FieldTaskDataElement",
position = 10,
persistenceType = TopComponent.PERSISTENCE_NEVER)
@NbBundle.Messages({
    "FieldTaskDataElement table column name value=Value",
    "FieldTaskDataElement table column name color=Color",
    "FieldTaskDataElement_display_name=Data"
})
public class FieldTaskDataElement extends BaseFieldTaskElement {

    private List<TreatmentZoneGroup> treatmentZoneGroups;
    private DataPanel dataPanel;
    private BodyPanel dataDecorator;
    private static final ImageIcon ICON = ImageUtilities.loadImageIcon("nl/cloudfarming/client/task/field/task32.png", true);

    public FieldTaskDataElement(Lookup lookup) {
        super(lookup);

        Map<Double, TreatmentZoneGroup> groups = new HashMap<>();
        for (TreatmentZone treatmentZone : fieldTask.getActivityField().getTreatmentZones()) {
            TreatmentZoneGroup group;
            if (!groups.containsKey(treatmentZone.getValue())) {
                group = new TreatmentZoneGroup(treatmentZone.getValue());
                groups.put(treatmentZone.getValue(), group);
            } else {
                group = groups.get(treatmentZone.getValue());
            }
            group.addTreatmentZone(treatmentZone);

        }
        this.treatmentZoneGroups = new ArrayList<>(groups.values());
        init();
    }

    public FieldTaskDataElement(final FieldTask fieldTask, final TreeSet<TreatmentZoneGroup> treatmentZoneGroups) {
        super(Lookups.fixed(fieldTask));
        this.treatmentZoneGroups = new ArrayList(treatmentZoneGroups);
        init();
    }

    @Override
    public void setName(String name) {
        super.setName(truncate(name));
    }

    private String truncate(String s) {
        final int max = 12;
        return (s == null || s.length() <= max) ? s : s.substring(0, max) + "...";
    }

    private void init() {
        setToolTipText(fieldTask.getName());
        setName(fieldTask.getName());

        fieldTask.addPropertyChangeListener(Task.PROP_NAME, new TaskNameListener());
        setLayout(new BorderLayout());
        dataPanel = new DataPanel();
        dataPanel.getNameField().getDocument().addDocumentListener(new TaskNameValidator());
        dataPanel.getDefaultValueField().getDocument().addDocumentListener(new DefaultTznValueValidator());
        dataDecorator = BodyPanel.create(dataPanel);
        dataDecorator.setTitle(truncate(fieldTask.getName()));
        dataDecorator.setIcon(ICON);
        add(dataDecorator);


        bind();
        fieldTask.setDefaultTznValue(fieldTask.getAverageTznValue());
        dataPanel.addTableModelListener(new ValueTableListener());
    }

    private void bind() {
        BindingGroup taskBindingGroup = new BindingGroup();
        JTableBinding tableBinding = SwingBindings.createJTableBinding(
                AutoBinding.UpdateStrategy.READ,
                treatmentZoneGroups,
                dataPanel.getValueTable());

        BeanProperty valueProperty = BeanProperty.create("value");
        JTableBinding.ColumnBinding valueColumnBinding = tableBinding.addColumnBinding(valueProperty);
        valueColumnBinding.setColumnName(Bundle.FieldTaskDataElement_table_column_name_value());
        valueColumnBinding.setColumnClass(Double.class);
        valueColumnBinding.setValidator(new PositiveDoubleValidator());
        BeanProperty colorProperty = BeanProperty.create("color");
        JTableBinding.ColumnBinding colorColumnBinding = tableBinding.addColumnBinding(colorProperty);
        colorColumnBinding.setColumnClass(Color.class);
        colorColumnBinding.setColumnName(Bundle.FieldTaskDataElement_table_column_name_color());
        taskBindingGroup.addBinding(tableBinding);

        ELProperty taskNameProperty = ELProperty.create("${text_ON_ACTION_OR_FOCUS_LOST}");
        BeanProperty taskPropName = BeanProperty.create(Task.PROP_NAME);
        Binding taskNameBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                fieldTask, taskPropName, dataPanel.getNameField(), taskNameProperty);
        taskNameBinding.setValidator(new NonEmptyStringValidator());
        taskBindingGroup.addBinding(taskNameBinding);

        Binding taskAverageValueBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ,
                fieldTask, BeanProperty.create(ExportableTask.PROP_AVERAGE_TZN_VALUE),
                dataPanel.getAverageValueField(), BeanProperty.create("text"));
        taskBindingGroup.addBinding(taskAverageValueBinding);
        
        Binding taskDefaultValueBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                fieldTask, BeanProperty.create(ExportableTask.PROP_DEFAULT_TZN_VALUE),
                dataPanel.getDefaultValueField(), ELProperty.create("${text_ON_ACTION_OR_FOCUS_LOST}"));
        taskDefaultValueBinding.setConverter(new DoubleConverter());
        taskDefaultValueBinding.setValidator(new PositiveDoubleValidator());
        taskBindingGroup.addBinding(taskDefaultValueBinding);

        taskBindingGroup.bind();
    }

    public DataPanel getDataPanel() {
        return dataPanel;
    }

    @Override
    public JComponent getVisualRepresentation() {
        return dataDecorator;
    }

    /**
     * TableModelListener used to update the Grid
     */
    private class ValueTableListener implements TableModelListener {

        @Override
        public void tableChanged(TableModelEvent tme) {
            fieldTask.setRegenerateGrid(true);
            fieldTask.recalculateAverageTznValue();
            saveTask();
        }

        private synchronized void saveTask() {
            FieldTaskManager fieldTaskManager = Lookup.getDefault().lookup(FieldTaskManager.class);
            fieldTaskManager.save(fieldTask);
        }
    }

    /**
     * Listens to name changes on the task and updates the tooltiptext the name
     * and the name of the AsbodyPanel
     */
    private class TaskNameListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent pce) {
            if (Task.PROP_NAME.equals(pce.getPropertyName())) {
                final String name = (String) pce.getNewValue();
                updateName();
                dataDecorator.setTitle(truncate(name));
            }
        }
    }

    /**
     * Class to validate the name of a task
     */
    @NbBundle.Messages("TaskNameValidator invalid name message=Name must be entered")
    private class TaskNameValidator extends UnifiedDocumentListener {

        @Override
        protected void handleUpdate(DocumentEvent de) {
            dataPanel.getNameRow().setValid();
            if (dataPanel.getNameField().getText().isEmpty()) {
                dataPanel.getNameRow().setInvalid(Bundle.TaskNameValidator_invalid_name_message());
            }
        }
    }

    private class DefaultTznValueValidator extends UnifiedDocumentListener {

        private final DoubleConverter converter = new DoubleConverter();
        private final List<Validator<Double>> validators = new ArrayList<Validator<Double>>() {

            {
                add(new DoubleValidator());
                add(new PositiveDoubleValidator());
            }
        };

        @Override
        protected void handleUpdate(DocumentEvent de) {
            dataPanel.getDefaultValueRow().setValid();
            String txtValue = dataPanel.getDefaultValueField().getText();
            if (!txtValue.isEmpty()) {
                Double doubleValue = converter.convertReverse(txtValue);
                for (Validator<Double> validator : validators) {
                    Validator.Result result = validator.validate(doubleValue);
                    if (result != null) {
                        dataPanel.getDefaultValueRow().setInvalid(result.getDescription());
                        return;
                    }
                }
            }
        }
    }

    @NbBundle.Messages({"DataPanel task name tooltip text=Fieldtask name",
        "DataPanel task AverageTznValue label=Average rate:",
        "DataPanel task AverageTznValue tooltip=Average of the distinct treatmentzone values",
        "DataPanel task defaultTznValue label=Loss of GPS rate:",
        "DataPanel task defaultTznValue tooltip=Default rate to use when the GPS is lost; if left empty the average rate will be used",})
    public class DataPanel extends JPanel {

        private static final int VALUE_SCROLLPANE_HEIGHT = 300;
        private JTable valueTable;
        private JTextField nameField;
        private FormRow nameRow;
        private JLabel averageValueField; // read-only
        private JTextField defaultValueField;
        private FormRow defaultValueRow;

        public DataPanel() {
            init();
        }

        private void init() {
            this.setLayout(new MigLayout("wrap 1"));
            setBackground(Color.white);

            nameField = new JTextField();
            nameField.setToolTipText(Bundle.DataPanel_task_name_tooltip_text());
            nameRow = new FormRow(true, nameField);

            valueTable = new JTable();
            ValueEditor cellEditor = new ValueEditor();

            valueTable.setDefaultEditor(Double.class, cellEditor);
            valueTable.setDefaultRenderer(Color.class, new ColorRenderer());

            JScrollPane valueScrollPane = new JScrollPane(valueTable);
            valueScrollPane.setPreferredSize(new Dimension(FormRow.FIELD_WIDTH_SMALL, VALUE_SCROLLPANE_HEIGHT));

            averageValueField = new JLabel();
            JLabel averageValueLabel = new JLabel(Bundle.DataPanel_task_AverageTznValue_label());
            averageValueField.setToolTipText(Bundle.DataPanel_task_AverageTznValue_tooltip());
            averageValueLabel.setToolTipText(Bundle.DataPanel_task_AverageTznValue_tooltip());
            FormRow averageValueRow = new FormRow(false, averageValueLabel, averageValueField);

            defaultValueField = new JTextField();
            JLabel defaultValueLabel = new JLabel(Bundle.DataPanel_task_defaultTznValue_label());
            defaultValueField.setToolTipText(Bundle.DataPanel_task_defaultTznValue_tooltip());
            defaultValueLabel.setToolTipText(Bundle.DataPanel_task_defaultTznValue_tooltip());
            defaultValueRow = new FormRow(false, defaultValueLabel, defaultValueField);

            this.add(nameRow);
            this.add(valueScrollPane);
            this.add(averageValueRow);
            this.add(defaultValueRow);
        }

        public Number getValueAt(int row, int column) {
            return (Number) valueTable.getValueAt(row, column);
        }

        public void setValueAt(Double value, int row, int column) {
            valueTable.setValueAt(value, row, column);
        }

        public JTable getValueTable() {
            return valueTable;
        }

        public JTextField getNameField() {
            return nameField;
        }

        public FormRow getNameRow() {
            return nameRow;
        }

        public JLabel getAverageValueField() {
            return averageValueField;
        }

        public JTextField getDefaultValueField() {
            return defaultValueField;
        }

        public FormRow getDefaultValueRow() {
            return defaultValueRow;
        }

        public void addTableModelListener(TableModelListener listener) {
            valueTable.getModel().addTableModelListener(listener);
        }

        public void removeTableModelListener(TableModelListener listener) {
            valueTable.getModel().removeTableModelListener(listener);
        }

        public class ValueEditor extends AbstractCellEditor
                implements TableCellEditor {

            private JTextField valueTextField;

            public ValueEditor() {
                valueTextField = new JTextField();
            }

            //Implement the one CellEditor method that AbstractCellEditor doesn't.
            @Override
            public Object getCellEditorValue() {
                return valueTextField.getText();
            }

            //Implement the one method defined by TableCellEditor.
            @Override
            public Component getTableCellEditorComponent(JTable table,
                    Object value,
                    boolean isSelected,
                    int row,
                    int column) {
                valueTextField = new JTextField(value.toString());
                valueTextField.setCaret(new FixedCaret());
                return valueTextField;
            }

            class FixedCaret extends DefaultCaret {

                @Override
                public void mousePressed(MouseEvent e) {
                    if (SwingUtilities.isLeftMouseButton(e)) {
                        positionCaret(e);
                        JTextComponent component = getComponent();
                        if (component.isEnabled() && !component.hasFocus()) {
                            // For editable text components, we perform some trickery...  
                            if (component.isEditable()) {
                                // first off, store the current caret position / selection:  
                                int savedDot = getDot();
                                int savedMark = getMark();
                                // Now request focus. This will cause all the text in the field to  
                                // be selected because of our focus listener...  
                                component.requestFocus();
                                // So now we reset the caret back the way it should be:  
                                setDot(savedMark);
                                moveDot(savedDot);
                            } // On  non-editable text components always select all, even  
                            // when you mouse click into them...  
                            else {
                                component.requestFocus();
                            }
                        }
                    }
                }

                @Override
                public void focusGained(FocusEvent e) {
                    super.focusGained(e);
                    // When focus is gained, select all the text  
                    int length = getComponent().getDocument().getLength();
                    setDot(0);
                    moveDot(length);
                }
            }
        }

        public class ColorRenderer extends JLabel implements TableCellRenderer {

            @Override
            public Component getTableCellRendererComponent(
                    JTable table, Object object,
                    boolean isSelected, boolean hasFocus,
                    int row, int column) {
                Color color = (Color) (object == null ? new Color(0.0f, 0.0f, 0.0f, 0.0f) : object);
                setOpaque(true);
                setBackground(color);
                setToolTipText("RGB: "
                        + color.getRed() + ", "
                        + color.getGreen() + ", "
                        + color.getBlue());
                return this;
            }
        }
    }
}
