/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.field;

import org.openide.util.ImageUtilities;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 * @author Maciek Dulkiewicz <maciek.dulkiewicz@ordina.nl>
 */
public class FieldTaskOverviewPanel extends javax.swing.JPanel {

    /**
     * Creates new form FieldTaskOverviewPanel
     */
    public FieldTaskOverviewPanel() {
        initComponents();
    }

    public void setTask(FieldTask task) {
        taskStatePanel1.setTask(task);
        workWidthPanel1.setTask(task);
        addActivityFieldPanel(task);
        taskExportButton1.setTask(task);
    }

    /**
     * Add an ActivityFieldPanel to the TaskContentPanel JTabbedPanel. The
     * ActivityFieldPanel handles it's own content but needs a task to do so.
     * The newly added ActivityFieldPanel also needs a pretty icon that is
     * displayed on it's own tab in the TaskContentPane
     *
     * @param task Task containing ActivityField objects
     */
    private void addActivityFieldPanel(FieldTask task) {
        taskContentPanel.add(new FieldTaskPanel(task));
        //components in the JTabbedPanel are accessed using an index
        //the index of the newly added component is the componentCount of the 
        //TaskContentPanel minus 1
        int newComponentIndex = taskContentPanel.getComponentCount() - 1;
        taskContentPanel.setIconAt(newComponentIndex, ImageUtilities.loadImageIcon(FieldTaskPanel.TAB_ICON_RESOURCE_PATH, false));
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        taskStatePanel1 = new nl.cloudfarming.client.task.windows.TaskStatePanel();
        taskContentPanel = new javax.swing.JTabbedPane();
        workWidthPanel1 = new nl.cloudfarming.client.task.field.WorkWidthPanel();
        taskExportButton1 = new nl.cloudfarming.client.task.field.TaskExportButton();

        taskContentPanel.setBackground(java.awt.Color.white);
        taskContentPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(org.openide.util.NbBundle.getMessage(FieldTaskOverviewPanel.class, "FieldTaskOverviewPanel.taskContentPanel.border.title"))); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(taskStatePanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(workWidthPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(taskExportButton1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(taskContentPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 221, Short.MAX_VALUE))
                        .addGap(9, 9, 9)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(taskStatePanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(workWidthPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(taskContentPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 358, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(taskExportButton1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTabbedPane taskContentPanel;
    private nl.cloudfarming.client.task.field.TaskExportButton taskExportButton1;
    private nl.cloudfarming.client.task.windows.TaskStatePanel taskStatePanel1;
    private nl.cloudfarming.client.task.field.WorkWidthPanel workWidthPanel1;
    // End of variables declaration//GEN-END:variables
}
