/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.field;

import java.util.List;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import nl.cloudfarming.client.field.model.ActivityField;
import nl.cloudfarming.client.field.model.TreatmentZone;
import nl.cloudfarming.client.util.swing.beanbinding.PositiveDoubleValidator;
import org.jdesktop.beansbinding.AutoBinding;
import org.jdesktop.beansbinding.BeanProperty;
import org.jdesktop.beansbinding.Converter;
import org.jdesktop.swingbinding.JTableBinding;
import org.jdesktop.swingbinding.JTableBinding.ColumnBinding;
import org.jdesktop.swingbinding.SwingBindings;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 * @author Wytse Visser
 * @author Maciek Dulkiewicz
 */
public class FieldTaskPanel extends javax.swing.JPanel implements TableModelListener {

    private JTableBinding binding = null;
    public static final String TAB_ICON_RESOURCE_PATH = "nl/cloudfarming/client/task/field/treatmentzone.png";
    private FieldTask task;

    /**
     * Creates new form ActivityFieldPanel
     */
    public FieldTaskPanel() {
        initComponents();
    }

    public FieldTaskPanel(FieldTask task) {
        this();
        setTask(task);
        treatmentZoneTable.getModel().addTableModelListener(this);
    }

    public final void setTask(FieldTask task) {
        this.task = task;
        if (binding != null) {
            binding.unbind();
        }
        ActivityField field = task.getLookup().lookup(ActivityField.class);
        List<TreatmentZone> treatmentZones = field.getTreatmentZones();
        if (!treatmentZones.isEmpty()) {
            binding = SwingBindings.createJTableBinding(
                    AutoBinding.UpdateStrategy.READ,
                    treatmentZones,
                    treatmentZoneTable);

            BeanProperty nameProperty = BeanProperty.create("name");
            ColumnBinding nameBinding = binding.addColumnBinding(nameProperty);
            nameBinding.setColumnName("Name");
            nameBinding.setEditable(false);

            BeanProperty valueProperty = BeanProperty.create("value");
            ColumnBinding columnBinding = binding.addColumnBinding(valueProperty);
            columnBinding.setColumnName("Value");
            columnBinding.setConverter(new StringToDoubleConverter());
            columnBinding.setValidator(new PositiveDoubleValidator());

            binding.bind();
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        arrayList1 = new java.util.ArrayList();
        jScrollPane2 = new javax.swing.JScrollPane();
        treatmentZoneTable = new javax.swing.JTable();

        jScrollPane2.setViewportView(treatmentZoneTable);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 375, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private java.util.ArrayList arrayList1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTable treatmentZoneTable;
    // End of variables declaration//GEN-END:variables


    @Override
    public void tableChanged(TableModelEvent e) {
        task.setRegenerateGrid(true);
        task.fireExternalPropertyChange(FieldTask.PROP_TREATMENTZONE_VALUE, null, null);
    }

    private class StringToDoubleConverter extends Converter<Double, String> {

        @Override
        public String convertForward(Double t) {
            return t + "";
        }

        @Override
        public Double convertReverse(String s) {
            Double d = Double.parseDouble(s);
            if(d == -0.0) {
                d = 0.0;
            }
            return d;
        }
        
    }
}
